import 'daterangepicker';
import 'tempusdominus-bootstrap-4';

const displayToast = (window as any).displayToast as (messageType: string, message:string) => void;

$(() => {
    ($("#date") as any ).daterangepicker({
        "timePicker": true,
        "locale": {
            "format": "YYYY-MM-DD-hh-mm",
            "separator": " → ",
            "applyLabel": "Appliquer",
            "cancelLabel": "Annuler",
            "fromLabel": "De",
            "toLabel": "À",
            "customRangeLabel": "Custom",
            "weekLabel": "S",
            "daysOfWeek": [
                "Di",
                "Lu",
                "Ma",
                "Me",
                "Je",
                "Ve",
                "Sa"
            ],
            "monthNames": [
                "Janvier",
                "Février",
                "Mars",
                "Avril",
                "Mai",
                "Juin",
                "Juillet",
                "Août",
                "Septembre",
                "Octobre",
                "Novembre",
                "Décembre"
            ],
            "firstDay": 0
        },
    });

    $('#date').on('change', function() {
        const inputPropArray: Array<string> = ["year", "month", "day", "hour", "minute" ];

        const startInput: Map<string, HTMLSelectElement> = new Map<string, HTMLSelectElement>();

        startInput.set( "year",  document.querySelector("#session_startDate_date_year"))
        startInput.set( "month",  document.querySelector("#session_startDate_date_month"))
        startInput.set( "day",  document.querySelector("#session_startDate_date_day"))
        startInput.set( "hour",  document.querySelector("#session_startDate_time_hour"))
        startInput.set( "minute", document.querySelector("#session_startDate_time_minute"))


        const endInput: Map<string, HTMLSelectElement> = new Map<string, HTMLSelectElement>();
        endInput.set( "year",  document.querySelector("#session_endDate_date_year"))
        endInput.set( "month",   document.querySelector("#session_endDate_date_month"))
        endInput.set( "day",  document.querySelector("#session_endDate_date_day"))
        endInput.set( "hour",  document.querySelector("#session_endDate_time_hour"))
        endInput.set( "minute",  document.querySelector("#session_endDate_time_minute"))

        if ( !startInput || !endInput ) {
            console.error("Could not find hidden inputs");
            return;
        }
        const input = this as HTMLInputElement;
        const values = input.value.split(' ');

        const startDateTime = values[0].split('-');
        const endDateTime = values[2].split('-');

        startDateTime.forEach( (value: string, index: number) => {
            const prop = inputPropArray[index];
            (startInput.get(prop) as HTMLSelectElement).value = parseInt(value).toString();
        } );
        endDateTime.forEach( (value: string, index: number) => {
            const prop = inputPropArray[index];
            (endInput.get(prop) as HTMLSelectElement).value = parseInt(value).toString();
        } );
    });

    $("#sessionAddForm").on('submit', (e) => {
        e.preventDefault();

        const form = e.currentTarget as HTMLFormElement;
        if (!form.checkValidity()) {
            form.classList.add("was-validated");
            return;
        }

        const $form = $(form);
        const submitButton = form.querySelector('#session_submit') as HTMLButtonElement;
        const oldText = submitButton.innerText;

        const spinner = document.createElement("div") as HTMLDivElement;
        spinner.classList.add("spinner-border");
        spinner.setAttribute("role", "status");
        const spinner_sr = document.createElement('span') as HTMLSpanElement;
        spinner_sr.classList.add("sr-only");
        spinner_sr.innerText = "Loading...";

        spinner.appendChild(spinner_sr);
        submitButton.innerHTML = "";
        submitButton.appendChild(spinner);
        submitButton.disabled = true;
        $.ajax({
            url: $form.attr('action'),
            method: $form.attr('method'),
            data: $form.serialize(),
            success: () => {
                displayToast("success", "La session a bien été créée");
                setTimeout(() => location.reload(), 1000);
            },
            error: () => {
                displayToast("error","Une erreur est survenue ! Veuillez réessayer ultérieurement.");
            },
            complete: () => {
                submitButton.disabled = false;
                submitButton.innerText = oldText;
            }
        });
    })
});
