<?php

namespace App\Command;

use App\Repository\SessionRepository;
use App\Service\ApplicationService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

#[AsCommand(
    name: 'app:admin:routes:recalculate',
    description: 'Recalculate all application routes for the current session',
)]
class AdminRoutesRecalculateCommand extends Command
{
    private SessionRepository $sessionRepository;
    private ApplicationService $applicationService;
    private EntityManagerInterface $manager;

    public function __construct(
        SessionRepository $sessionRepository,
        ApplicationService $applicationService,
        EntityManagerInterface $manager
    ) {
        parent::__construct();
        $this->sessionRepository = $sessionRepository;
        $this->applicationService = $applicationService;
        $this->manager = $manager;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);

        $activeSession = $this->sessionRepository->findActive();
        if ($activeSession === null) {
            $io->warning('There is no session currently active');
            return Command::INVALID;
        }

        $parkingApplications = $activeSession->getParkingApplications();

        $success = 0;
        $total = count($parkingApplications);
        for ($i = 0; $i < $total; $i++) {
            $io->write("Processing application " . $i + 1 . "/${total}\n");
            $parkingApp = $parkingApplications[$i];

            $oldTravelInfo = $parkingApp->getTravelInfo();
            $this->applicationService->setParkingApp($parkingApp);
            $res = $this->applicationService->calculateRoute();
            if ($res) {
                $this->manager->remove($oldTravelInfo);
            }
            $success += $res;
        }

        $this->manager->flush();
        $io->success("The routes have been recalculated ! The success rate is ${success}/${total}.");

        return Command::SUCCESS;
    }
}
