<?php

namespace App\Command;

use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use App\Repository\ParkingApplicationRepository;
use App\Entity\EmailAccessToken;
use App\Service\EmailTokenService;
use Doctrine\ORM\EntityManagerInterface;

#[AsCommand(
    name: 'app:email-token:generate',
    description: 'Generates a temporary login token for any user',
)]
class EmailTokenGenerateCommand extends Command
{
    private ParkingApplicationRepository $parkingApplicationDirectory;
    private EmailTokenService $emailTokenService;

    public function __construct(ParkingApplicationRepository $parkingApplicationDirectory, EmailTokenService $emailTokenService)
    {
        Command::__construct();
        $this->parkingApplicationDirectory = $parkingApplicationDirectory;
        $this->emailTokenService = $emailTokenService;
    }

    protected function configure(): void
    {
        $this
            ->addArgument('user-email', InputArgument::REQUIRED, 'Email of user')
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $email = $input->getArgument('user-email');

        $application = $this->parkingApplicationDirectory->findOneBy(["email" => $email]);
        if ($application === null) {
            $io->error("Aucune application avec cet email existe");
            return Command::FAILURE;
        }

        $emailAccessToken = $this->emailTokenService->generate($application);

        $io->success('Success ! The token is ' . $emailAccessToken->getToken());

        return Command::SUCCESS;
    }
}
