<?php

namespace App\Controller\Admin;

use Symfony\Component\HttpFoundation\File\File as HTTPFile;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use App\Entity\File;
use App\Entity\ParkingApplication;
use App\Enum\ApplicationStatusEnum;
use App\Repository\ParkingApplicationRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Exception\BadRequestException;
use Symfony\Component\HttpFoundation\File\Exception\FileNotFoundException;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Validator\Validator\ValidatorInterface;

use Symfony\Component\Mime\Address;
use App\Enum\EmailEnum;
use App\Service\MailerService;

#[Route('/admin/application', name: 'admin_application')]
class ApplicationController extends AbstractController
{
    # Default get of all applications, sorted by time
    #[Route('/', name: '', methods: ['GET'])]
    public function application(ParkingApplicationRepository $applicationRepo): Response
    {
        $data = [];
        $counts = [];
        foreach (ApplicationStatusEnum::cases() as $value) {
            $values = $applicationRepo->findByStatus($value->value);
            array_push($data, ...$values);
            $counts[$value->value] = count($values);
        }
        return $this->render('admin/application/index.html.twig', ['applications' => $data, "counts" => $counts]);
    }

    #[Route('/{id}/status', name: '_change_status', methods: ['PUT'])]
    public function changeStatus(
        ParkingApplication $parkingApp,
        ValidatorInterface $validator,
        EntityManagerInterface $manager,
        MailerService $mailer,
        Request $request
    ): Response {
        $params = json_decode($request->getContent(), true);

        if (!is_array($params)) {
            throw new BadRequestException("Invalid JSON !");
        }
        if (!array_key_exists("status", $params)) {
            throw new BadRequestException("status not in JSON !");
        }

        $newStatus = ApplicationStatusEnum::tryFrom($params['status']);
        if ($newStatus == null) {
            throw new BadRequestException("Incorrect status !");
        }

        $parkingApp->setStatus($newStatus);
        if (count($validator->validate($parkingApp)) != 0)
            throw new BadRequestException("Parking Application is invalid !");

            $mailer->sendEmail(
                new Address($parkingApp->getEmail()),
                [
                    "application" => $parkingApp
                ],
                EmailEnum::APPLICATION_PROCESSED_EMAIL
            );

        $manager->persist($parkingApp);
        $manager->flush();
        return new Response();
    }

    #[Route('/{id}', name: '_get', methods: ['GET'])]
    public function getApplication(ParkingApplication $parkingApp): Response
    {
        return $this->render('admin/application/get.html.twig', [
            'application' => $parkingApp
        ]);
    }

    #[Route('/file/{id}', name: '_get_file', methods: ['GET'])]
    public function getFile(File $file): Response
    {
        $dirname = $this->getParameter('uploads_dir');
        try {
            $fileObj = new HTTPFile($dirname . $file->getFileName());
        }
        catch (FileNotFoundException $e) {
            throw $this->createNotFoundException("File not found");
        }
        return $this->file($fileObj, disposition: ResponseHeaderBag::DISPOSITION_INLINE);
    }

    #[Route('/file/validate/{id}', name: '_validate_file', methods: ['PUT'])]
        public function verifyFile(File $file, EntityManagerInterface $manager): Response
    {
        $file->validate();
        $manager->persist($file);
        $manager->flush();
        return new Response();
    }
}
