<?php

namespace App\DataFixtures;

use App\Entity\File;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Persistence\ObjectManager;

use App\Entity\ParkingApplication;
use App\Entity\Session;
use App\Entity\TravelInfo;
use App\Entity\Vehicle;
use App\Enum\ApplicationStatusEnum;
use App\Enum\AssociationEnum;
use App\Enum\VehicleTypeEnum;

class ApplicationFixtures extends Fixture
{
    public function load(ObjectManager $manager): void
    {
        $RANDOM_NAMES = ["Gary", "Bob", "Adem", "Romain", "Christophe", "Isa", "Arthur", "David", "Pierre", "Yves", "Pierre-Yves", "Nom", "non", "Random", "Test", "Crane", "Basilic", "Close", "Sun", "Moment", "Dreams", "Absatou", "Donald", "Trump", "Corinne", "Eric", "Zemmour", "Jean", "Vincent", "Jourel", "Aber Aouni", "Lebbadi-Breteau", "Faneva", "Sapin", "Noël", "Liam", "Noah", "Olivier", "Elijah", "William", "James", "Benjamin", "Lucas", "Henry", "Alexander", "Olivia", "Emma", "Ava", "Charlotte", "Sophia", "Amelia", "Isabella", "Mia", "Evelyn", "Harper", "White", "Black", "Blue", "Orange", "STEP", "Gay"];
        $nb_random_names = count($RANDOM_NAMES) - 1;
        $nb_status = count(ApplicationStatusEnum::cases()) - 1;
        $nb_veh_type = count(VehicleTypeEnum::cases()) - 1;
        $nb_asso = count(AssociationEnum::cases()) - 1;
        $session = new Session();
        $timezone = new \DateTimeZone('America/Toronto');
        $session->setStartDate(new \DateTime("2021-12-20", $timezone));
        $session->setEndDate(new \DateTime("2022-12-30", $timezone));
        $session->setName("SmallSession");
        $manager->persist($session);

        for ($i = 0; $i < 10; $i++) {
            $application = new ParkingApplication();
            $application->setForename($RANDOM_NAMES[random_int(0, $nb_random_names)]);
            $application->setSurname($RANDOM_NAMES[random_int(0, $nb_random_names)]);
            $application->setEmail($application->getForename() . "." . $application->getSurname() . "@polymtl.ca");
            $application->setStudentId(random_int(1000000, 2000000));
            $application->setAddress("1234 Avenue de Montréal\nQuébec, France");

            $travelInfo = new TravelInfo();
            $travelInfo->setDistance(random_int(10, 10000) / 10);
            $travelInfo->setDurationDriving(random_int(20, 500000));
            $travelInfo->setDurationDrivingTraffic($travelInfo->getDurationDriving() + random_int(20, 500));
            $travelInfo->setDurationTransit(random_int(20, 500000));
            $application->setTravelInfo($travelInfo);

            $application->setStatus(ApplicationStatusEnum::cases()[random_int(0, $nb_status)]);

            $nb_address_proofs = random_int(1, 4);
            for ($j = 0; $j < $nb_address_proofs; $j++) {
                $file = new File();
                $file->setFileName("main" . random_int(4, 3000) . ".pdf");
                $application->addAddressProof($file);
            }

            $nb_vehicles = random_int(1, 4);
            for ($j = 0; $j < $nb_vehicles; $j++) {
                $vehicle = new Vehicle();
                $vehicle->setManufacturer($RANDOM_NAMES[random_int(0, $nb_random_names)]);
                $vehicle->setColour($RANDOM_NAMES[random_int(0, $nb_random_names)]);
                $vehicle->setModel($RANDOM_NAMES[random_int(0, $nb_random_names)] . " " . $RANDOM_NAMES[random_int(0, $nb_random_names)]);
                $vehicle->setType(VehicleTypeEnum::cases()[random_int(0, $nb_veh_type)]);
                $vehicle->setYear(random_int(1980, 2022));
                $vehicle->setLicensePlate("AG-" . random_int(1000, 9999) . "-YU");
                $application->addVehicle($vehicle);
            }

            $carpool = (bool)random_int(0, 1);
            $application->setCarpooling($carpool);
            if ($carpool) {
                $description = "";
                $nb_words =  random_int(10, 30);
                for ($j = 0; $j < $nb_words; $j++) {
                    $description = $description. ' ' . $RANDOM_NAMES[random_int(0, $nb_random_names)];
                }
                $application->setCarpoolDescription($description);
            }

            $parent = (bool)random_int(0, 1);
            $application->setParent($parent);
            if ($parent) {
                $nb_parent_proofs = random_int(1, 6);
                for ($j = 0; $j < $nb_parent_proofs; $j++) {
                    $file = new File();
                    $file->setFileName("parent" . random_int(4, 3000) . ".pdf");
                    $application->addParentProof($file);
                }
            }

            $application->setAssociation(AssociationEnum::cases()[random_int(0, $nb_asso)]);

            $physical = (bool)random_int(0, 1);
            $application->setPhysicalCondition($physical);
            if ($physical) {
                $nb_physical_proofs = random_int(1, 6);
                for ($j = 0; $j < $nb_physical_proofs; $j++) {
                    $file = new File();
                    $file->setFileName("phys" . random_int(4, 3000) . ".pdf");
                    $application->addPhysicalConditionProof($file);
                }
            }
            $application->setSession($session);

            $manager->persist($application);
        }
        $manager->flush();
    }
}
