<?php

namespace App\Entity;

use App\Repository\EmailAccessTokenRepository;
use Doctrine\ORM\Mapping as ORM;

#[ORM\Entity(repositoryClass: EmailAccessTokenRepository::class)]
class EmailAccessToken
{
    #[ORM\Id]
    #[ORM\GeneratedValue]
    #[ORM\Column(type: "integer")]
    private $id;

    #[ORM\Column(type: "string", length: 32)]
    private $token;

    #[ORM\Column(type: "datetime")]
    private $createdDate;

    #[ORM\Column(type: "datetime")]
    private $expirationDate;

    #[ORM\ManyToOne(targetEntity: ParkingApplication::class, inversedBy: "emailAccessTokens")]
    #[ORM\JoinColumn(nullable: false)]
    private $application;

    public function __construct(ParkingApplication $application = NULL)
    {
        $this->setApplication($application);
        $this->setToken(bin2hex(random_bytes(16)));
        $this->setCreatedDate(new \DateTime());
        $this->setExpirationDate(new \DateTime("+10 minutes"));
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getToken(): ?string
    {
        return $this->token;
    }

    public function setToken(string $token): self
    {
        $this->token = $token;

        return $this;
    }

    public function getCreatedDate(): ?\DateTimeInterface
    {
        return $this->createdDate;
    }

    public function setCreatedDate(\DateTimeInterface $createdDate): self
    {
        $this->createdDate = $createdDate;

        return $this;
    }

    public function getExpirationDate(): ?\DateTimeInterface
    {
        return $this->expirationDate;
    }

    public function setExpirationDate(\DateTimeInterface $expirationDate): self
    {
        $this->expirationDate = $expirationDate;

        return $this;
    }

    public function getApplication(): ?ParkingApplication
    {
        return $this->application;
    }

    public function setApplication(?ParkingApplication $application): self
    {
        $this->application = $application;

        return $this;
    }
}
