<?php

namespace App\Repository;

use App\Entity\ParkingApplication;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;
use Doctrine\Common\Collections\Criteria;

/**
 * @method ParkingApplication|null find($id, $lockMode = null, $lockVersion = null)
 * @method ParkingApplication|null findOneBy(array $criteria, array $orderBy = null)
 * @method ParkingApplication[]    findAll()
 * @method ParkingApplication[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class ParkingApplicationRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, ParkingApplication::class);
    }

    public function findOrderedByTime()
    {
        return $this->createQueryBuilder('p')
            ->orderBy('p.travelTime', Criteria::DESC)
            ->getQuery()
            ->getResult();
    }

    public function findByStatus(string $status)
    {
        return $this->createQueryBuilder('p')
            ->join('p.session', 's')
            ->andWhere('p.status = :status')
            ->andWhere('s.archived = 0')
            ->setParameter('status', $status)
            ->getQuery()
            ->getResult();
    }

    public function findByStatusOrderedTime(string $status)
    {
        return $this->createQueryBuilder('p')
            ->andWhere('p.status = :status')
            ->setParameter('status', $status)
            ->orderBy('p.travelTime', Criteria::DESC)
            ->getQuery()
            ->getResult();
    }

    public function findByEmailOrderedByCreationDate(string $email)
    {
        return $this->createQueryBuilder('p')
            ->Where('p.email = :email')
            ->setParameter('email', $email)
            ->orderBy('p.createdDate', Criteria::DESC)
            ->getQuery()
            ->getResult();
    }
}
