<?php

namespace App\Repository;

use App\Entity\Session;
use Doctrine\Bundle\DoctrineBundle\Repository\ServiceEntityRepository;
use Doctrine\Persistence\ManagerRegistry;

/**
 * @method Session|null find($id, $lockMode = null, $lockVersion = null)
 * @method Session|null findOneBy(array $criteria, array $orderBy = null)
 * @method Session[]    findAll()
 * @method Session[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class SessionRepository extends ServiceEntityRepository
{
    public function __construct(ManagerRegistry $registry)
    {
        parent::__construct($registry, Session::class);
    }

    public function findCurrent() : ?Session
    {
        return $this->createQueryBuilder('s')
            ->andWhere('s.archived = 0')
            ->andWhere('s.startDate < CURRENT_TIMESTAMP()')
            ->andWhere('CURRENT_TIMESTAMP() < s.endDate')
            ->getQuery()
            ->getOneOrNullResult();
    }

    public function findActive() : ?Session
    {
        return $this->createQueryBuilder('s')
            ->andWhere('s.archived = 0')
            ->getQuery()
            ->getOneOrNullResult();
    }

    public function findAllOrderedByRecent(): array
    {
        return $this->createQueryBuilder('s')
            ->orderBy("s.startDate", "DESC")
            ->getQuery()
            ->getResult();
    }

    public function findByDateIntersection(array $dates): ?array
    {
        $entityManager = $this->getEntityManager();

        return $entityManager->createQuery(
            'SELECT s
            FROM App\Entity\Session s
            WHERE (:startDate >= s.startDate AND :startDate <= s.endDate) OR
                  (:endDate >= s.startDate AND :endDate <= s.endDate) OR
                  (:startDate <= s.startDate AND :endDate >= s.endDate)'
            )
            ->setParameter("startDate", $dates["startDate"])
            ->setParameter("endDate", $dates["endDate"])
            ->getResult();
    }

    public function findActiveOrNothing(array $archived): ?array
    {
        if ($archived["archived"]) return null;

        return $this->createQueryBuilder('s')
            ->andWhere('s.archived = :archived')
            ->setParameter('archived', $archived["archived"])
            ->getQuery()
            ->getResult();
    }
}
