<?php

namespace App\Service;

use Symfony\Component\Mailer\MailerInterface;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Component\Mime\Address;
use App\Enum\EmailEnum;
use Symfony\Component\Mailer\Exception\TransportExceptionInterface;

class MailerService
{
    private MailerInterface $mailer;

    public function __construct(MailerInterface $mailer)
    {
        $this->mailer = $mailer;
    }

    public function sendEmail(Address|string $destinationEmail, array $context, EmailEnum $emailType): bool
    {
        $email = (new TemplatedEmail())
        ->from($_ENV["FROM_EMAIL"])
        ->to($destinationEmail)
        ->subject($this->chooseEmailSubject($emailType))
        ->htmlTemplate('emails/'.$emailType->value.'.html.twig')
        ->context($context);

        try
        {
            $this->mailer->send($email);
            return true;
        }
        catch (TransportExceptionInterface $e)
        {
            \Sentry\captureException($e);
            return false;
        }
    }

    private function chooseEmailSubject(EmailEnum $emailType): string
    {
        switch ($emailType) {
            case EmailEnum::ACTION_SUCCESS_EMAIL:
                return 'Mise à jour concernant votre demande de stationnement';
            case EmailEnum::APPLICATION_CONFIRMATION_EMAIL:
                return 'Confirmation de votre demande de stationnement';
            case EmailEnum::APPLICATION_PROCESSED_EMAIL:
                return 'L\'état de votre demande a été modifié';
            case EmailEnum::CONSULT_EDIT_APPLICATION_EMAIL:
                return 'Consulter ou modifier votre demande de stationnement';
        }
        return '';
    }
}
